from pytrevl.component import Component
from typing import List

class TrevlCodeGenerator:
    def __init__(self, component: Component):
        self.component = component

    def get_trevl(self) -> dict:
        yaml = {
            'description': 'Placeholder',
            'parameters': [],
            'components': [
                {
                    'id': self.component.id,
                    'type': 'chart',
                    'queries': self._create_queries(),
                    'display': self._create_display()
                }
            ]
        }
        return yaml

    def _create_queries(self) -> List[dict]:
        queries = [{
            'measures': [self.component.cube.name + "." + self.component.measure],
            'dimensions': [self.component.cube.name + "." + self.component.dimension],
            'filters': self._create_filters()
        }]
        return queries

    def _create_filters(self) -> List[dict]:
        filters = []
        for filter in self.component.filters:
            filters.append({
                'member': self.component.cube.name + "." + filter.variable,
                'operator': filter.operator,
                'values': [filter.value]
            })
        return filters

    def _create_display(self) -> dict:
        display = {
            'chart': {'type': self.component.type},
            'title': {'text': self.component.style.title},
            'series': [{
                'name': "$" + self.component.cube.name + "." + self.component.dimension,
                'x': "$" + self.component.cube.name + "." + self.component.dimension,
                'y': "$" + self.component.cube.name + "." + self.component.measure
            }]
        }
        if self.component.__class__.__name__ == 'DonutChart':
            del display['series'][0]['x']
            display['series'][0]['innerSize'] = self.component.style.innerSize

        return display


import html
import json

class IpythonHC:
    # template for the IFrame
    iframe_template = """
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <link href="https://www.highcharts.com/highslide/highslide.css" rel="stylesheet" />
    <script type="text/javascript" src="https://ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js"></script>
    <script type="text/javascript" src="https://code.highcharts.com/6/highcharts.js"></script>
    <script type="text/javascript" src="https://code.highcharts.com/6/highcharts-more.js"></script>
    <script type="text/javascript" src="https://code.highcharts.com/6/modules/heatmap.js"></script>
    <script type="text/javascript" src="https://code.highcharts.com/6/modules/exporting.js"></script>
  </head>
  <body style="margin:0;padding:0">
    <div id="container" style="">Loading....</div>
    <script>
    console.log('A');
      $(function(){{
        console.log('AA');
        Highcharts.setOptions({{ "global": {{}}, "lang": {{}}, }});
        var options = {options};
        var chart = new Highcharts.Chart('container', options);
        console.log('B'); 
      }});
    console.log('BB');
    </script>
  </body>
</html>
    """
    def __init__(self, options):
        self.options = options
        
    @property
    def iframe(self):
        return self.iframe_template.format(options=json.dumps(self.options))
        
    def _repr_html_(self):
        src_doc = html.escape(self.iframe).replace('\n', ' ')
        return ''.join((
            "<p><p>",
            f"""<iframe width=800 height=400 srcdoc="{src_doc}"></iframe>""",
            "<p><p>",
        ))



from typing import List
import yaml

from pytrevl.chart import Chart
from pytrevl.cube import Cube
from pytrevl.style import Style

from pytrevl.api.x_middle import X_Middle
from pytrevl.render import IpythonHC

class PieChart(Chart):
    '''
    PieChart component
    '''
    def __init__(self, id: str, cube: Cube, measure: str, dimension: str, filters: List, style: Style):
        self.id = id
        self.cube = cube
        self.measure = measure
        self.dimension = dimension
        self.filters = filters
        self.style = style
        self.type = "pie"

class DonutChart(Chart):
    '''
    DonutChart component
    '''
    def __init__(self, id: str, cube: Cube, measure: str, dimension: str, filters: List, style: Style):
        self.id = id
        self.cube = cube
        self.measure = measure
        self.dimension = dimension
        self.filters = filters
        self.style = style
        self.type = "pie"

class BarChart(Chart):
    '''
    BarChart component
    '''
    def __init__(self, id: str, cube: Cube, measure: str, dimension: str, filters: List, style: Style):
        self.id = id
        self.cube = cube
        self.measure = measure
        self.dimension = dimension
        self.filters = filters
        self.style = style
        self.type = "column"

class LineChart(Chart):
    '''
    LineChart component
    '''
    def __init__(self, id: str, cube: Cube, measure: str, dimension: str, filters: List, style: Style):
        self.id = id
        self.cube = cube
        self.measure = measure
        self.dimension = dimension
        self.filters = filters
        self.style = style
        self.type = "line"

class CustomChart:
    def __init__(self, json: dict):
        self.json = json
        self.type = ""

    def get_json(self) -> dict:
        return self.json
    
    def get_yaml(self) -> str:
        yaml_code = yaml.safe_dump(self.json)
        print(yaml_code)

    def show(self):
        x_middle_api = X_Middle(self)
        chart = x_middle_api.get_chart(json=self.json)
        render = IpythonHC(chart)
        return render

    def get_data(self) -> list:
        x_middle_api = X_Middle(self)
        data = x_middle_api.get_data(json=self.json)
        return data


'''
Handling Cube connections
'''
import os
import sqlalchemy
import pandas as pd

# Get global env variables for SQL - connection


class Cube:
    '''
    Cube api
    '''
    def __init__(self, x: str, y: str, cube: str, filters: dict):
        self.x = x
        self.y = y
        self.cube = cube
        self.filters = filters
        self.drivername=os.getenv("DRIVERNAME")
        self.username=os.getenv("USERNAME")
        self.password=os.getenv("PASSWORD")
        self.host=os.getenv("HOST")
        self.port=os.getenv("PORT")
        self.database=os.getenv("DATABASE")

    def connect(self) -> None:
        # Creating engine
        self.engine = sqlalchemy.create_engine(
                    sqlalchemy.engine.url.URL(
                        drivername=self.drivername,
                        username=self.username,
                        password=self.password,
                        host=self.host,
                        port=self.port,
                        database=self.database,
                    ),
                    echo_pool=True,
        )

        print("Connecting with Cube...")

        # Connecting to engine
        self.connection = self.engine.connect()

    def get_data(self) -> pd.DataFrame:
        # Converting Plot filters to SQL format
        self.filters_list = []
        for self.filter in self.filters.items():
            self.filters_list.append(self.filter[0] + " = '" + self.filter[1] + "'")
        self.filters_sql = " AND ".join(self.filters_list)

        # Building Cube query
        self.query = f"SELECT {self.x}, {self.y} FROM {self.cube} WHERE {self.filters_sql};"

        # Receiving Data
        df = pd.read_sql_query(self.query, self.connection) # Applies SQL query and returns a DataFrame

        return df


import yaml

from typing import List

from pytrevl.api.x_middle import X_Middle
from pytrevl.trevl_code_generator import TrevlCodeGenerator
from pytrevl.render import IpythonHC


class Chart:
    '''
    Chart parent class
    '''
        
    def get_json(self) -> dict:
        trevl_code_generator = TrevlCodeGenerator(self)
        yaml_code = trevl_code_generator.get_trevl()
        return yaml_code
    
    def get_yaml(self) -> str:
        trevl_code_generator = TrevlCodeGenerator(self)
        trevl_code = trevl_code_generator.get_trevl()
        yaml_code = yaml.safe_dump(trevl_code)
        print(yaml_code)

    def show(self):
        x_middle_api = X_Middle(self)
        chart = x_middle_api.get_chart()
        render = IpythonHC(chart)
        return render

    def get_data(self) -> list:
        x_middle_api = X_Middle(self)
        data = x_middle_api.get_data()
        return data



'''
Handling Cube connections
'''
import os
import sqlalchemy
import pandas as pd
from typing import List
from pytrevl import Cube, Filter

# Get global env variables for SQL - connection

SQL_CUBE_OPERATORS = {
    "equals": " = ",
    ">=": " >= ",
    "<=": " <= "
"""
Add more in the future
"""
}


class CubeQuery:
    '''
    Cube api
    '''
    def __init__(self, queries: List, cube: Cube, filters: List = None):
        self.queries = queries
        self.cube = cube
        self.filters = filters
        self.drivername=os.getenv("DRIVERNAME")
        self.username=os.getenv("USERNAME")
        self.password=os.getenv("PASSWORD")
        self.host=os.getenv("HOST")
        self.port=os.getenv("PORT")
        self.database=os.getenv("DATABASE")

    def get_data(self) -> pd.DataFrame:
        connection = self._connect()

        query_sql = ", ".join(self.queries)

        if self.filters:
            filters_list = []
            for filter in self.filters:
                filters_list.append(filter.variable + SQL_CUBE_OPERATORS.get(filter.operator) + "'" + filter.value + "'")
            filters_sql = " AND ".join(filters_list)

            # Building Cube query
            query = f"SELECT {query_sql} FROM {self.cube.name} WHERE {filters_sql};"
        else:
            query = f"SELECT {query_sql} FROM {self.cube.name};"

        # Receiving Data
        df = pd.read_sql_query(query, connection) # Applies SQL query and returns a DataFrame

        return df

    def _connect(self) -> None:
        # Creating engine
        engine = sqlalchemy.create_engine(
                    sqlalchemy.engine.url.URL(
                        drivername=self.drivername,
                        username=self.username,
                        password=self.password,
                        host=self.host,
                        port=self.port,
                        database=self.database,
                    ),
                    echo_pool=True,
        )

        print("Connecting with Cube...")

        # Connecting to engine
        connection = engine.connect()
        return connection



import requests
import os
import pandas as pd

from pytrevl.component import Component
from pytrevl.trevl_code_generator import TrevlCodeGenerator

class X_Middle:
    '''
    X-Middle api
    '''
    def __init__(self, component: Component):
        self.component = component
        self.http_auth = self.authenticate()

    def authenticate(self):
        x_username = os.getenv("X_MIDDLE_USERNAME")
        x_password = os.getenv("X_MIDDLE_PASSWORD")
        http_auth = requests.auth.HTTPBasicAuth(x_username, x_password)
        return http_auth

    def post_request(self, chart_json):
        base_url = os.getenv("X_MIDDLE_BASEURL")
        req = {"abstractConfig": chart_json,}
        response = requests.post(f'{base_url}/dashboards', auth=self.http_auth, json=req)
        
        if "error" in response.json():
            raise Exception("X-MIDDLE API ERROR RESPONSE: " + response.json()["error"] + "  REQUEST: " + str(req))
        
        return response
    
    def get_chart(self, json: dict = None) -> dict:
        # Create a YAML object from the TREVL code
        if not json:
            chart_json = self.get_trevl()
        else:
            chart_json = json
        # Post a request to the X-Middle API and receive a response
        x_middle_response = self.post_request(chart_json=chart_json)
        
        # Extract the JSON information from the response
        highcharts_json = x_middle_response.json()["components"][0]

        return highcharts_json

    def get_trevl(self):
        travel_code_generator = TrevlCodeGenerator(self.component)
        trevl_code = travel_code_generator.get_trevl()
        return trevl_code

    def get_data(self, json: dict = None):
        # Get Chart JSON from X-Middle
        if not json:
            highcharts_json = self.get_chart()
        else:
            highcharts_json = self.get_chart(json)
            self.component.type = json["components"][0]['display']['chart']['type']
        
        # Create lists for data and category names
        data = []
        category_names = []

        if self.component.type == "pie":
            '''
            If component type is 'pie' then x value will be called 'name' upon return from X-Middle
            '''
            for data_points in highcharts_json["series"][0]["data"]:
                data.append([data_points["name"], data_points["y"]])

        elif self.component.type == "column":
            '''
            If component type is 'column' then x value is the names of options/catagories
            '''
            category_names = highcharts_json["xAxis"]["categories"]

            for counter, value in enumerate(highcharts_json["series"][0]["data"]):
                data.append([category_names[counter],value[1]])

        elif self.component.type == "line":
            '''
            If component type is 'line' then x value is 1st and y ist 2nd in list of data_points
            '''
            for data_points in highcharts_json["series"][0]["data"]:
                data.append([data_points[0], data_points[1]])

        else:
            raise Exception(f"Component type {self.component.type} is not defined.")
       
        # Create DataFrame
        df = pd.DataFrame(data, columns=["X", "Y"])
        df = df.set_index("X")

        return df


import yaml

from pytrevl.component import Component

class Dashboard:
    def __init__(self, description: str = ""):
        self.components: dict[str, Component] = {}
        self.description = description

    def add_component(self, component: Component) -> str:
        self.components[component.id] = component
        return component.id

    def remove_component(self, component: Component) -> str:
        self.components.pop(component.id)
        return f"{component.id} removed from dashboard"

    def get_component(self, component_id: str) -> Component:
        return self.components[component_id]

    def show(self) -> None:
        for component in self.components.values():
            display(component.show()) # Render Charts in Jupyter
    
    def get_json(self) -> dict:
        dashboard_json = {'description': self.description, 'parameters': [], 'components': []}
        for component in self.components.values():
            json = component.get_json()
            dashboard_json["components"].append(json["components"][0])
        return dashboard_json

    def get_yaml(self) -> str:
        dashboard_json = self.get_json()
        dashboard_yaml = yaml.safe_dump(dashboard_json)
        print(dashboard_yaml)


'''
Import modules to shorten import path to use i.e.:

pytrevl.Dashboard()

instead of

pytrevl.dashboard.Dashboard()
'''

from .dashboard import Dashboard
from .components import PieChart, BarChart, LineChart, DonutChart, CustomChart
from .cube import Cube
from .filter import Filter
from .style import Style
from .api.cube_query import CubeQuery


# Version and Author

__version__ = '0.2.0'
__author__ = 'Trendence Institut'